/*
 * Copyright (C) 2000-2001 Damien Douxchamps  <douxchamps@ieee.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "activate.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "connect_signals.h"
#include "update.h"
#include <libdc1394/dc1394_control.h>
#include <libraw1394/raw1394.h>
#include "video1394.h"

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *configure_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern dc1394_camerainfo *camera;
extern dc1394_miscinfo *misc_info;
extern dc1394_feature_set *feature_set;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];
extern char* trigger_mode_list[4];
extern char* channel_num_list[16];
extern int camera_num;
extern int current_camera;

void ActivateCapabilities()
{
  // ----------- Commander window --------------------------------------------
  
  // basic capabilities:
  ActivatePowerFrame();

  // trigger capabilities:
  ActivateTriggerFrame();

  // format capabilities:
  BuildFormatMenu();

  // camera menu:
  BuildCameraMenu();

  // update FPS menu according to format:
  UpdateFpsMenu();

  // memory capabilities
  ActivateMemoryFrame();

  // iso flow buttons:
  gtk_widget_set_sensitive(lookup_widget(capture_window,"iso_frame"),TRUE);
  gtk_widget_set_sensitive(lookup_widget(capture_window,"iso_start"),TRUE);
  gtk_widget_set_sensitive(lookup_widget(capture_window,"iso_restart"),TRUE);
  gtk_widget_set_sensitive(lookup_widget(capture_window,"iso_stop"),TRUE);

  // Activate ranges (min, max, current, capabilities)
  ActivateRange(porthole_window, FEATURE_FOCUS);
  ActivateRange(porthole_window, FEATURE_PAN);
  ActivateRange(porthole_window, FEATURE_TILT);
  ActivateRange(porthole_window, FEATURE_ZOOM);

  ActivateRange(aperture_window, FEATURE_EXPOSURE);
  ActivateRange(aperture_window, FEATURE_IRIS);
  ActivateRange(aperture_window, FEATURE_SHUTTER);
  ActivateRange(aperture_window, FEATURE_GAIN);
  ActivateRange(aperture_window, FEATURE_OPTICAL_FILTER);

  ActivateRange(color_window, FEATURE_BRIGHTNESS);
  ActivateRange(color_window, FEATURE_GAMMA);
  ActivateRange(color_window, FEATURE_SATURATION);
  ActivateRange(color_window, FEATURE_HUE);
  ActivateRange(color_window, FEATURE_WHITE_BALANCE);
  ActivateRange(color_window, FEATURE_SHARPNESS);

  ActivateRange(capture_window, FEATURE_CAPTURE_SIZE);
  ActivateRange(capture_window, FEATURE_CAPTURE_QUALITY);

  ActivateRange(temperature_window, FEATURE_TEMPERATURE);
  
  // Connect range signals since Glade 0.5 doesn't handle that (can you imagine this??)
  ConnectRangeSignals();
}


void BuildFormatMenu()
{
  int err;
  quadlet_t modes, formats;

  // formats capabilities:
  err= dc1394_query_supported_formats(camera->handle, camera->id, &formats);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_0"),formats & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_1"),formats & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_2"),formats & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_6"),formats & (0x1<<25) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_7"),formats & (0x1<<24) );

  // corresponding modes:
  err= dc1394_query_supported_modes(camera->handle, camera->id, 0, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode5"),modes & (0x1<<26) );
  
  err= dc1394_query_supported_modes(camera->handle, camera->id, 1, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode5"),modes & (0x1<<26) );
  
  err= dc1394_query_supported_modes(camera->handle, camera->id, 2, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode5"),modes & (0x1<<26) );

  err= dc1394_query_supported_modes(camera->handle, camera->id, 6, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f6_mode0"),modes & (0x1<<31) );

  err= dc1394_query_supported_modes(camera->handle, camera->id, 7, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode5"),modes & (0x1<<26) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode6"),modes & (0x1<<25) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode7"),modes & (0x1<<24) );

}

void ActivateRange(GtkWidget* current_window, int feature)
{
  GtkAdjustment *adjustment, *adjustment2;

  if (feature_set->feature[feature].available)
    {
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_frame_list[feature]),TRUE);
      // one push auto:
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_op_list[feature]),
			       feature_set->feature[feature].one_push);
      // on/off switching:
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_power_list[feature]),
			       feature_set->feature[feature].on_off_capable);
      // manual mode:
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_man_list[feature]),
			       feature_set->feature[feature].manual_capable);
      // auto mode:
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_auto_list[feature]),
			       feature_set->feature[feature].auto_capable);

      // values:
      adjustment=(GtkAdjustment*)gtk_adjustment_new(feature_set->feature[feature].min,
						    feature_set->feature[feature].min,
						    feature_set->feature[feature].max,1,10,0);
      switch(feature)
	{
	  case FEATURE_WHITE_BALANCE:
	    adjustment2=(GtkAdjustment*)gtk_adjustment_new(feature_set->feature[feature].min,
							   feature_set->feature[feature].min,
							   feature_set->feature[feature].max,1,10,0);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "whitebal_BU_scale"),adjustment);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "whitebal_RV_scale"),adjustment2);break;
	  case FEATURE_TEMPERATURE:
	    adjustment2=(GtkAdjustment*)gtk_adjustment_new(feature_set->feature[feature].min,
							   feature_set->feature[feature].min,
							   feature_set->feature[feature].max,1,10,0);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "temp_goal_scale"),adjustment);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "temp_current_scale"),adjustment2);break;
	  default:
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, feature_scale_list[feature]),adjustment);
	}
    }
    
  UpdateRange(current_window,feature);
}

void
BuildTriggerModeMenu()
{
  int err, i, modes;
  quadlet_t value;
  int k=0;
  int index[NUM_TRIGGER_MODE];

  GtkWidget* trigger_mode;
  GtkWidget* trigger_mode_menu;
  GtkWidget* glade_menuitem;

  err=dc1394_query_feature_characteristics(camera->handle,camera->id,FEATURE_TRIGGER,&value);

  modes=( (value & (0xF << 12))>>12 );
  gtk_widget_destroy(GTK_WIDGET (lookup_widget(commander_window,"trigger_mode"))); // remove previous menu

  trigger_mode = gtk_option_menu_new ();
  gtk_widget_ref (trigger_mode);
  gtk_object_set_data_full (GTK_OBJECT (commander_window), "trigger_mode", trigger_mode,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (trigger_mode);
  gtk_table_attach_defaults (GTK_TABLE (lookup_widget(commander_window,"table17")), trigger_mode, 0, 2, 1, 2);
  gtk_container_set_border_width (GTK_CONTAINER (trigger_mode), 1);

  trigger_mode_menu = gtk_menu_new ();

  if (modes) // at least one mode present
    { // external trigger available:
      for (i=0;i<=TRIGGER_MODE_MAX;i++)
	{
	  if (modes & (0x1<<(TRIGGER_MODE_MAX-i)))
	    {
	      index[i]=k;
	      k++;
	      glade_menuitem = gtk_menu_item_new_with_label (_(trigger_mode_list[i]));
	      gtk_widget_show (glade_menuitem);
	      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
	      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
				  GTK_SIGNAL_FUNC (on_trigger_mode_activate),
				  (int*)i);
	    }
	  else
	    index[i]=0;
	}
    }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (trigger_mode), trigger_mode_menu);

  // sets the active menu item:
  gtk_option_menu_set_history (GTK_OPTION_MENU (trigger_mode), index[feature_set->feature[FEATURE_TRIGGER].trigger_mode]);
}


void
BuildMemoryChannelMenu()
{
  int i;
  // note: this function does not require indexing the menu items for they are always in the same consecutive order
  GtkWidget* channel_num;
  GtkWidget* channel_num_menu;
  GtkWidget* glade_menuitem;

  misc_info->load_channel=0; // should be moved to update function (?)
  misc_info->save_channel=0;

  gtk_widget_destroy(GTK_WIDGET (lookup_widget(commander_window,"memory_channel"))); // remove previous menu

  channel_num = gtk_option_menu_new ();
  gtk_widget_ref (channel_num);
  gtk_object_set_data_full (GTK_OBJECT (commander_window), "memory_channel", channel_num,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (channel_num);
  gtk_table_attach_defaults (GTK_TABLE (lookup_widget(commander_window,"table16")), channel_num, 0, 1, 0, 1);
  gtk_container_set_border_width (GTK_CONTAINER (channel_num), 1);

  channel_num_menu = gtk_menu_new ();

  for (i=0;i<=misc_info->mem_channel_number;i++)
    {
      glade_menuitem = gtk_menu_item_new_with_label (_(channel_num_list[i]));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (channel_num_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_memory_channel_activate),
			  (int*)i); // i is an int passed in a pointer variable. This is 'normal'.
    }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (channel_num), channel_num_menu);

  // sets the active menu item:
  gtk_option_menu_set_history (GTK_OPTION_MENU (channel_num), misc_info->load_channel);

}

void
ActivateTriggerFrame()
{
  GtkAdjustment *adjustment;
  
  // always set the trigger frame on (because it contains the fps menu):
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_frame"),TRUE);

  // check external trigger feature presence:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_external"),
			   feature_set->feature[FEATURE_TRIGGER].available);

  // polarity change available:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_polarity"),
			   (feature_set->feature[FEATURE_TRIGGER].available) &&
			   (feature_set->feature[FEATURE_TRIGGER].polarity_capable));

  // build external trigger modes menu:  
  BuildTriggerModeMenu();
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_mode"),
			   (feature_set->feature[FEATURE_TRIGGER].available) &&
			   (feature_set->feature[FEATURE_TRIGGER].is_on));

  // set the trigger_count value adjustment
  adjustment=(GtkAdjustment*)gtk_adjustment_new(0,1,(int)0xFFF,1,10,0);// max. number for trigger parameter is 12bit=FFFh
  gtk_spin_button_set_adjustment((GtkSpinButton*)lookup_widget(commander_window, "trigger_count"),adjustment);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_count"),
			   (feature_set->feature[FEATURE_TRIGGER].available) &&
			   (feature_set->feature[FEATURE_TRIGGER].is_on) && 
			   (feature_set->feature[FEATURE_TRIGGER].trigger_mode >> 1)); // 2 or 3 means 2nd bit=1 => >>1==1

}


void ActivatePowerFrame()
{
  quadlet_t basic_funcs;
  int err;
  // these two functions are always present:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"power_frame"),TRUE);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"power_reset"),TRUE);

  // activate if camera capable of power on/off:
  err=dc1394_query_basic_functionality(camera->handle,camera->id,&basic_funcs);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"power_on"),(basic_funcs & 0x1<<16));
  gtk_widget_set_sensitive(lookup_widget(commander_window,"power_off"),(basic_funcs & 0x1<<16));
}


void ActivateMemoryFrame()
{
  // load always present, so we can activate it:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"memory_frame"),TRUE);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"load_mem"),TRUE);

  // save not activated by default (it is not avail. for factory defaults channel):
  gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(commander_window,"save_mem")), FALSE);

  // activate the mem channel menu:
  BuildMemoryChannelMenu();
  gtk_widget_set_sensitive(lookup_widget(commander_window,"memory_channel"),TRUE);

}


void
BuildCameraMenu()
{
  int i;
  // note: this function does not require indexing the menu items for they are always in the same consecutive order
  GtkWidget* camera_id;
  GtkWidget* camera_id_menu;
  GtkWidget* glade_menuitem;

  gtk_widget_destroy(GTK_WIDGET (lookup_widget(commander_window,"camera_select"))); // remove previous menu

  camera_id = gtk_option_menu_new ();
  gtk_widget_ref (camera_id);
  gtk_object_set_data_full (GTK_OBJECT (commander_window), "camera_select", camera_id,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (camera_id);
  gtk_table_attach_defaults (GTK_TABLE (lookup_widget(commander_window,"table9")), camera_id, 1, 2, 0, 1);
  gtk_container_set_border_width (GTK_CONTAINER (camera_id), 1);

  camera_id_menu = gtk_menu_new ();

  for (i=0;i<camera_num;i++)
    {
      glade_menuitem = gtk_menu_item_new_with_label (_(camera->model));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (camera_id_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_camera_select_activate),
			  (int*)i); // i is an int passed in a pointer variable. This is 'normal'.
    }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (camera_id), camera_id_menu);

  // sets the active menu item:
  gtk_option_menu_set_history (GTK_OPTION_MENU (camera_id) , current_camera);

}

